/*
 Code by lingib 
 Last update 16 October 2017
 
 ----------
 Theory
 ---------- 
 The image is sub-divided into cells.
 
 The image is then sampled at each cell location,  a "value" in the 
 range [0..255] is assigned to that cell.

 This grayscale number is then converted to a gcode brush-width.
 
 ----------
 Copyright 
 ----------
 This code is free software: you can redistribute it and/or
 modify it under the terms of the GNU General Public License as published
 by the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This software is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License. If
 not, see <http://www.gnu.org/licenses/>.
 */

// -------------------------------------
// declarations
// -------------------------------------
PImage src;                                             //declare variables of type PImage
PrintWriter output;                                     //instantiate "output" for printing files

int
  cellWidth = 8,                                       //cell width in pixels ... determines the detail
  cellHeight = cellWidth, //make cell square
  columns, 
  rows;

boolean
  penUp=true;                                           //flag

// -------------------------------------
// setup
// -------------------------------------
void setup() {
  size(467, 620);                                       //image.jpg dimensions (cellWidth=5)

  background(255);                                      //screen image background white

  rows=height/cellHeight;                               //number of cell rows
  columns=width/cellWidth;                              //number of cell columns

  src = loadImage("image.jpg");                         //get image.jpg
  src.filter(GRAY);                                     //convert image to monochrome

  output = createWriter("image.ngc");                   //open a file for storing the g-code

  noLoop();                                             //main loop only runs once
}

// -------------------------------------
// main loop
// -------------------------------------
void draw() {
  //image(src, 0, 0);                                   //superimpose image
  drawImage();                                          //draw the image

  // ----- close the g-code file
  output.println("G00 X0 Y0");                          //home
  output.flush();                                       //write the remaining data to the file
  output.close();                                       //finishes the file
  //exit();                                             //stop the program
}

// ------------------------------------
// drawImage
// ------------------------------------

/*
  The function map(value, 0, 256, 0, 16) produces a 16-step grayscale
 */
void drawImage() {

  float value;

  // ----- fill each cell with a pattern
  for (int row=0; row<rows; row++) {
    penUp=true;                                          //raise pen at the end of each row    
    for (int column=0; column<columns; column++) {
      value=averageValue(column, row);                   //value is in the range 0..255
      paint(column, row, map(value, 0, 255, 100, 0));    //convert brush value to range 0..100% paint
    }
  }
}

// ------------------------------------
// averageValue
// ------------------------------------
float averageValue(int column, int row) {

  float 
    sum=0.0; 

  int 
    x=column*cellWidth, 
    y=row*cellHeight, 
    pixel, 
    count=0;


  for (int j=0; j<cellHeight; j++) {                     //for each cell row
    for (int i=0; i<cellWidth; i++) {                    //add each cell column
      pixel=(x+i) + (y+j)*width;
      sum+=brightness(src.pixels[pixel]);
      count++;
    }
  }

  if (count<1) count=1; 
  return sum/count;                                       //average cell value is in range 0..255
}


// ------------------------------------
// PAINT()
// ------------------------------------
void paint(int column, int row, float brush_width) {

  // ----- preprocessing
  float x=(float)column*cellWidth;
  float y=(float)row*cellHeight;

  // ----- create g-code
  if (penUp) {
    output.println("G00 X" + (x) + " Y" + (height-y) );                           //display gcode as file created
    println("G00 X" + (x) + " Y" + (height-y) );                                  //paint the first cell in the row
    penUp=false;
  } else {
    for (int i=0; i<cellWidth; i++) {    
      output.println("G01 X" + (x+i) + " Y" + (height-y) + " BW" + brush_width);  //display gcode as file created
      println("G01 X" + (x+i) + " Y" + (height-y) + " BW" + brush_width);         //paint the remaining cells
    }
    noStroke();
    fill(map(brush_width, 0, 100, 255, 0));
    rect(x, y, cellWidth, cellHeight);
  }
}